package fr.janua.client;

import java.util.Set;

import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.proxy.UserProxy;

/**
 * Singleton pour gérer l'état de la connexion (connecté/déconnecté), et méthodes d'accès rapide aux principales fonctionnalités (exemple : nom de groupe, droits, ...).
 * Exemple d'utilisation : <code>UserManager.getInstance.getCurrentUser()</code>
 * @author gilles
 *
 */
public class UserManager {
	/**
	 * Instance du singleton.
	 * 
	 * @see UserManager#getInstance()
	 */
	private static UserManager instance;

	/**
	 * L'utilisateur courant. Doit être passé à <code>null</code> pour l'état déconnecté.
	 * 
	 * @see UserManager#getCurrentUser()
	 * @see UserManager#setCurrentUser(UserProxy)
	 */
	private UserProxy currentUser;

	/**
	 * Singleton implique aucun constructeur.
	 */
	private UserManager() {}

	/**
	 * Créer et retourne une instance de la classe à la première utilisation, ou retourne l'instance de classe courante. 
	 * @return
	 */
	public static UserManager getInstance() {
		if (null == instance) {
			instance = new UserManager();
		}
		return instance;
	}


	/* Accesseurs / Mutateurs */

	/**
	 * Définit l'utilisateur courant. Passer <code>null</code> en paramètre pour l'état déconnecté.
	 * @param currentUser the currentUser to set
	 */
	public void setCurrentUser(UserProxy currentUser) {
		this.currentUser = currentUser;
	}

	/**
	 * @return the currentUser
	 */
	public UserProxy getCurrentUser() {
		return currentUser;
	}

	/* Raccourcis */

	/**
	 * Retourne le login de l'utilisateur, ou null si on n'est pas connecté.
	 * @return
	 */
	public String getLogin(){
		if(currentUser != null)
			return currentUser.getLogin();
		else
			return null;
	}
	public String getPassword(){
		if(currentUser != null)
			return currentUser.getPassword();
		else
			return null;
	}

	
	/**
	 * Retourne un booléen selon l'état de la connexion (connecté ou déconnecté)
	 * @return
	 */
	public boolean isLogged() {
		if(currentUser == null)
			return false;
		else
			return true;
	}
	
	public long getLastActivity() {
		if(currentUser != null)
			return currentUser.getLastActivity();
		else
			return 0;
	}

	public GroupProxy getGroup() {
		if(currentUser != null)
			return currentUser.getGroup();
		else
			return null;
	}

	public String getGroupName(){
		if (getGroup() != null)
			return getGroup().getName();
		else
			return "";
	}

	/**
	 * Retourne l'Union entre les SitesAllowed de l'user et les SitesAllowed du groupe.
	 * @return
	 */
	public Set<SiteProxy> getSitesAllowed(){
		if(getGroup() != null){
			Set<SiteProxy> res = getGroup().getSiteAllowed();
			res.addAll(currentUser.getSiteAllowed());
			return res;
		}else{
			return null;
		}
	}

	/**
	 * Retourne les TypesAllowed du groupe.
	 * @return
	 */
	public Set<TypeProxy> getTypesAllowed(){
		if(getGroup() != null){
			return getGroup().getTypeAllowed();
		}else{
			return null;
		}
	}

	/* Gestion accès pages */
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir un profil.
	 * @return
	 */
	public boolean canAccessPageProfil(){
		if(!isLogged() || currentUser.getGroup().getCrudUser() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir un groupe.
	 * @return
	 */
	public boolean canAccessPageGroup(){
		if(!isLogged() || currentUser.getGroup().getCrudGroup() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir un site.
	 * @return
	 */
	public boolean canAccessPageSite(){
		if(!isLogged() || currentUser.getGroup().getCrudSite() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir un type.
	 * @return
	 */
	public boolean canAccessPageType(){
		if(!isLogged() || currentUser.getGroup().getCrudType() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir un document.
	 * @return
	 */
	public boolean canAccessPageAdminDocument() {
		if(!isLogged() || currentUser.getGroup().getCrudUploadDoc() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de voir les logs.
	 * @return
	 */
	public boolean canAccessPageLog(){
		if(!isLogged() || currentUser.getGroup().getCrudUploadDoc() == 0)
			return false;
		else{
			return true;
		}
	}
	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de voir d'uploader des documents.
	 * @return
	 */
	public boolean canAccessPageUploadDoc() {
		if(!isLogged() || currentUser.getGroup().getCrudUploadDoc() == 0)
			return false;
		else{
			return true;
		}
	}

	/**
	 * Retourne <code>true</code> si l'utilisateur a le droit de créer, éditer ou voir : les profils ET les groupes ET les sites ET les types ET les logs.
	 * @return
	 */
	public boolean canAccessAdministration(){
		return canAccessPageProfil() || canAccessPageGroup() || canAccessPageSite() || canAccessPageType() || canAccessPageLog() || canAccessPageUploadDoc();
	}

	/* Gestion droits : Message */
	/**
	 * Retourne true si l'utilisateur courant peut créer des messages.
	 * @return
	 */
	public boolean isCreatableMessage(){
		if(isLogged()){
			return isCreatable(currentUser.getGroup().getCrudMessage());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut editer des messages.
	 * @return
	 */
	public boolean isEditableMessage(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudMessage());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut voir les messages.
	 * @return
	 */
	public boolean isViewableMessage(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudMessage());
		}else{
			return false;
		}
	}


	/* Gestion droits : Profil */
	/**
	 * Retourne true si l'utilisateur courant peut créer des profils.
	 * @return
	 */
	public boolean isCreatableUser(){
		if(isLogged()){
			return isCreatable(currentUser.getGroup().getCrudUser());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut editer des profils.
	 * @return
	 */
	public boolean isEditableUser(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudUser());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut voir les profils.
	 * @return
	 */
	public boolean isViewableUser(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudUser());
		}else{
			return false;
		}
	}


	/* Gestion droits : Group */
	/**
	 * Retourne true si l'utilisateur courant peut créer des groupes.
	 * @return
	 */
	public boolean isCreatableGroup(){
		if(isLogged()){
			return isCreatable(currentUser.getGroup().getCrudGroup());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut editer des groupes.
	 * @return
	 */
	public boolean isEditableGroup(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudGroup());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut voir les groupes.
	 * @return
	 */
	public boolean isViewableGroup(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudGroup());
		}else{
			return false;
		}
	}

	/* Gestion droits : Site */
	/**
	 * Retourne true si l'utilisateur courant peut créer des sites.
	 * @return
	 */
	public boolean isCreatableSite(){
		if(isLogged()){
			return isCreatable(currentUser.getGroup().getCrudSite());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut editer des sites.
	 * @return
	 */
	public boolean isEditableSite(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudSite());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut voir les sites.
	 * @return
	 */
	public boolean isViewableSite(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudSite());
		}else{
			return false;
		}
	}

	/* Gestion droits : Type */
	/**
	 * Retourne true si l'utilisateur courant peut créer des types.
	 * @return
	 */
	public boolean isCreatableType(){
		if(isLogged()){
			return isCreatable(currentUser.getGroup().getCrudType());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut editer des types.
	 * @return
	 */
	public boolean isEditableType(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudType());
		}else{
			return false;
		}
	}
	/**
	 * Retourne true si l'utilisateur courant peut voir les types.
	 * @return
	 */
	public boolean isViewableType(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudType());
		}else{
			return false;
		}
	}
	
	/* Gestion droits : Admin Document */
	public boolean isEditableDocument(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudUploadDoc());
		}else{
			return false;
		}
	}
	/* Gestion droits : Log */
	/**
	 * Retourne true si l'utilisateur courant peut voir les logs.
	 * @return
	 */
	public boolean isViewableLog(){
		if(isLogged()){
			return isViewable(currentUser.getGroup().getCrudLog());
		}else{
			return false;
		}
	}
	
	/* Gestion droits : Upload doc */
	/**
	 * Retourne true si l'utilisateur courant peut envoyer des fichiers.
	 * @return
	 */
	public boolean isEditableUploadDoc(){
		if(isLogged()){
			return isEditable(currentUser.getGroup().getCrudUploadDoc());
		}else{
			return false;
		}
	}


	/* Gestion droits : conversion */
	/**
	 * Renvoie true si le numero correspond au droit "Créer"
	 * @param nbGroup
	 * @return
	 */
	private boolean isCreatable(int nbGroup){
		if(nbGroup>3)
			return true;
		return false;
	}
	/**
	 * Renvoie true si le numero correspond au droit "Editer"
	 * @param nbGroup
	 * @return
	 */
	private boolean isEditable(int nbGroup){
		if(nbGroup == 2 || nbGroup == 3 || nbGroup > 5)
			return true;
		return false;
	}
	/**
	 * Renvoie true si le numero correspond au droit "Voir"
	 * @param nbGroup
	 * @return
	 */
	private boolean isViewable(int nbGroup){
		if(nbGroup%2 == 1)
			return true;
		return false;
	}

	
	

	






}
